searchData={"items":[{"type":"behaviour","title":"SimpleSchema","doc":"# SimpleSchema\n\nSimpleSchema は JSON の検証と各データ構造への設定を行うライブラリです。\n\n- [hex.pm](https://hex.pm/packages/simple_schema)","ref":"SimpleSchema.html"},{"type":"behaviour","title":"動機 - SimpleSchema","doc":"HTTP の API サーバを書いていると、よく HTTP の POST リクエストで JSON を受け取ることがありますが、この値が正しいフォーマットかどうかを検証したいことがあります。\n検証だけなら、[JSON Schema](http://json-schema.org/) を使うという手があり、幸いなことに Elixir には JSON Schema を実装した [ExJsonSchema](https://github.com/jonasschmidt/ex_json_schema) というライブラリがあります。\n\nしかし、JSON Schema は手で書くのが大変です。もう少し機能を絞った単純なスキーマを使いたいところです。\n\nまた、JSON Schema は検証しか行わないため、データにアクセスするのに手間が掛かります。\n\n```elixir\njson = Poison.decode!(conn.body_param)\n:ok = validate(json)\n\nhp = json[\"player\"][\"hp\"]\n# → json.player.hp と書きたい\n\ndatetime = json[\"datetime\"]                       # 文字列を取り出して\n{:ok, datetime, _} = DateTime.from_iso8601(value) # DateTime 型に変換する\n# → json.datetime した段階で DateTime 型であって欲しい\n```\n\nこのように、特に `DateTime` に変換するといった処理が必要な場合、検証と同時に変換まで済ませたいのです。\n\nそこで、簡単にスキーマを書けるようして、検証を行い、それらのデータを変換するライブラリとして、SimpleSchema というライブラリを作りました。","ref":"SimpleSchema.html#module-%E5%8B%95%E6%A9%9F"},{"type":"behaviour","title":"使い方 - SimpleSchema","doc":"以下のように使います。\n\n```elixir\n# defschema/1 を使ってスキーマを定義する\ndefmodule Person do\n  import SimpleSchema, only: [defschema: 1]\n\n  defschema [\n    name: :string,\n    age: {:integer, minimum: 0},\n  ]\nend\n\n# JSON 文字列をデコードしたデータを…\njson = %{\n  \"name\" => \"John Smith\",\n  \"age\" => 42,\n}\n\n# Person と一緒に from_json!/2 すると、Person 構造体に値が設定される\nperson = SimpleSchema.from_json!(Person, json)\n\nassert person.name == \"John Smith\"\nassert person.age == 42\n```\n\nこのように、`defschema/1` でスキーマを定義して、`SimpleSchema.from_json!/2` にそのスキーマと JSON オブジェクトを渡すと、JSON オブジェクトを検証し、指定したスキーマにデータを入れてくれます。\n\nJSON オブジェクトが `Person` スキーマを満たしていない場合、以下のようにエラーがでます。\n\n```elixir\nbad_json = %{\n  \"name\" => 100, # 文字列ではない\n  \"age\" => -10, # 無効な年齢\n  \"__additional_key__\" => 0, # 余分なキー\n}\n\n# from_json/2 は失敗する\n{:error, reason} = SimpleSchema.from_json(Person, bad_json)\nIO.inspect reason\n```\n\n出力:\n\n```\n[{\"Expected the value to be >= 0\", \"#/age\"},\n {\"Type mismatch. Expected String but got Integer.\", \"#/name\"},\n {\"Schema does not allow additional properties.\", \"#/__additional_key__\"}]\n```\n\nこの `Person` を内包するスキーマを定義することもできます。\nつまりスキーマはネスト可能です。\n\n```elixir\n# Person を内包する Group\ndefmodule Group do\n  import SimpleSchema, only: [defschema: 1]\n\n  defschema [\n    group_name: :string,\n    persons: [Person],\n  ]\nend\n\njson = %{\n  \"group_name\" => \"A Group\",\n  \"persons\" => [%{\n    \"name\" => \"John Smith\",\n    \"age\" => 42,\n  }, %{\n    \"name\" => \"YAMADA Taro\",\n    \"age\" => 20,\n  }],\n}\n\ngroup = SimpleSchema.from_json!(Group, json)\n\nassert group.group_name == \"A Group\"\nassert Enum.fetch!(group.persons, 1).age == 20\n```\n\n`Group` スキーマを `defschema/1` で定義していますが、`:group_name` が文字列であり、`:persons` が `Person` の配列であることが、見ればすぐに分かるでしょう。\n\nこれによって、共通するスキーマに名前を付けて再利用することができます。","ref":"SimpleSchema.html#module-%E4%BD%BF%E3%81%84%E6%96%B9"},{"type":"behaviour","title":"シンプルスキーマ - SimpleSchema","doc":"もう少し詳細に SimpleSchema の機能を説明します。\n\nSimpleSchema ライブラリが定義している、`SimpleSchema.from_json/2` の第１引数に渡せるスキーマのことを **シンプルスキーマ** と呼びます。\nJSON Schema と比べると大分単純で直感的な構文になっているので「シンプル」と名付けています。\n\n例えば、`:integer` はシンプルスキーマです。\n\n```elixir\nvalue = SimpleSchema.from_json!(:integer, 10)\nassert value == 10\n```\n\n`:integer` シンプルスキーマは、渡された値が整数であるかを確認し、整数であればその値を戻り値にします。\n整数に制約を付け加えることも可能です。\n\n```elixir\nvalue = SimpleSchema.from_json!({:integer, minimum: 10, maximum: 20}, 5)\n# RuntimeError: [{\"Expected the value to be >= 10\", \"#\"}]\n```\n\n`{:integer, opts}` という書き方もシンプルスキーマになります。\nこれは渡された値が整数であり、かつ10から20の範囲内であるかを確認し、正しければその値を戻り値にします。\n\n`%{}` という書き方もシンプルスキーマであり、各フィールドには、更にシンプルスキーマを渡すことができます。\n\n```elixir\nschema = %{\n  value: {:integer, optional: true},\n  point: %{\n    x: :integer,\n    y: :integer,\n  },\n}\ndata = %{\n  \"point\" => %{\n    \"x\" => 10,\n    \"y\" => 20,\n  }\n}\nvalue = SimpleSchema.from_json!(schema, data)\n# value == %{point: %{x: 10, y: 20}}\nassert value.point.x == 10\nassert value.point.y == 20\n```\n\nこのシンプルスキーマは、渡された値がマップであるかを確認し、渡された値の各フィールドが、指定したシンプルスキーマのフィールドと合っているかどうか確認します。\n正しければ、渡されたマップのキーを atom にした上で戻り値にします。\n\nまた、`:value` フィールドに `optional: true` という制約を付与しました。\nこれはマップのフィールドに渡すシンプルスキーマのみに指定可能で「このフィールドが無くてもエラーにしない」という意味になります。\nそのため `data` に `\"value\"` キーが存在していなくても `SimpleSchema.from_json!/2` が成功しています。\n\n#","ref":"SimpleSchema.html#module-%E3%82%B7%E3%83%B3%E3%83%97%E3%83%AB%E3%82%B9%E3%82%AD%E3%83%BC%E3%83%9E"},{"type":"behaviour","title":"シンプルスキーマの一覧 - SimpleSchema","doc":"シンプルスキーマは、以下のいずれかである必要があります。\n\n- `:boolean` または `{:boolean, opts}`\n- `:integer` または `{:integer, opts}`\n- `:number` または `{:number, opts}`\n- `:null` または `{:null, opts}`\n- `:string` または `{:string, opts}`\n- `:any` または `{:any, opts}`\n- `%{...}` または `{%{...}, opts}`\n- `[...]` または `{[...], opts}`\n- `SimpleSchema` ビヘイビアを実装したモジュール、または `{Module, opts}`\n\n`opts` には各制約をキーワードリストで指定します。\n\n#","ref":"SimpleSchema.html#module-%E3%82%B7%E3%83%B3%E3%83%97%E3%83%AB%E3%82%B9%E3%82%AD%E3%83%BC%E3%83%9E%E3%81%AE%E4%B8%80%E8%A6%A7"},{"type":"behaviour","title":"制約の一覧 - SimpleSchema","doc":"制約の一覧は以下の通りです。\n\n- `{:nullable, boolean}`: もし `true` なら `nil` を許可する。`:null` 以外のシンプルスキーマに指定可能。\n- `{:minimum, integer}`: 最小値。`:integer` と `:number` に指定可能。\n- `{:maximum, integer}`: 最大値。`:integer` と `:number` に指定可能。\n- `{:min_items, non_neg_integer}`: 最小の要素数。`:array` に指定可能。\n- `{:max_items, non_neg_integer}`: 最大の要素数。`:array` に指定可能。\n- `{:unique_items, boolean}`: もし `true` なら配列がユニークであることを要求される。`:array` に指定可能。\n- `{:min_length, non_neg_integer}`: 最小の長さ。`:string` に指定可能。\n- `{:max_length, non_neg_integer}`: 最大の長さ。`:string` に指定可能。\n- `{:enum, [...]}`: 要素に指定可能な値のリスト。`:integer` と `:string` に指定可能。\n- `{:format, :datetime | :email}`: 事前に定義されたフォーマットで検証する。`:string` に指定可能。\n- `{:optional, boolean}`: もし `true` なら、`%{...}` の子要素として必須では無い。`%{...}` の子要素のみ指定可能。\n- `{:tolerant, boolean}`: もし `true` なら、生成される JSON Schema に `\"additionalProperties\"` が設定される。つまり子要素に指定されてない要素を許可するようになる。`%{...}` に指定可能。デフォルトは `false`。\n- `{:default, any}`: フィールドのデフォルト値。渡された JSON にこのフィールドが存在しなかった場合はこの値になる。`%{...}` の子要素のみ指定可能。\n- `{:field, string}`: 対応する JSON のフィールド名。`%{...}` の子要素のみ指定可能。\n\n#","ref":"SimpleSchema.html#module-%E5%88%B6%E7%B4%84%E3%81%AE%E4%B8%80%E8%A6%A7"},{"type":"behaviour","title":"メタ情報 - SimpleSchema","doc":"シンプルスキーマや制約以外の情報は `opts` に `:meta` キーを使って追加します。\n\n```elixir\nschema = %{\n  value: {:integer, optional: true},\n  point: %{\n    x: {:integer, meta: %{description: \"x座標\"}},\n    y: {:integer, meta: %{description: \"y座標\"}},\n  },\n}\n```\n\n## `SimpleSchema` ビヘイビア\n\n`SimpleSchema` ビヘイビアを実装したモジュールは、シンプルスキーマになります。\nこれを使うことで、特定のスキーマに名前を付けたり、特定の構造体に変換できるようになります。\n\n例えば、`\"2017-11-27T11:49:50+09:00\"` といった ISO 8601 に従った日付を `DateTime` 型として取得するには、以下のように定義します。\n\n```elixir\ndefmodule DateTimeSchema do\n  @behaviour SimpleSchema\n\n  @impl SimpleSchema\n  def schema(_opts) do\n    {:string, format: :datetime}\n  end\n\n  @impl SimpleSchema\n  def from_json(_schema, value, _opts) do\n    case DateTime.from_iso8601(value) do\n      {:ok, datetime, _} -> {:ok, datetime}\n      {:error, reason} -> {:error, reason}\n    end\n  end\n\n  @impl SimpleSchema\n  def to_json(_schema, value, _opts) do\n    {:ok, DateTime.to_iso8601(value)}\n  end\nend\n```\n\n`DateTimeSchema` は `SimpleSchema` ビヘイビアを実装しているためシンプルスキーマになります。\nそのため、以下のように `SimpleSchema.from_json!/2` に渡すことができます。\n\n```elixir\ndatetime = SimpleSchema.from_json!(DateTimeSchema, \"2017-11-27T11:49:50+09:00\")\n# datetime == #DateTime<2017-11-27 02:49:50Z>\n```\n\nこのように `SimpleSchema` ビヘイビアを実装することで、特定のスキーマに `DateTimeSchema` というを付け、`DateTime` 型の構造体に変換して利用できるようになります。\nなお、上記の `DateTimeSchema` に相当する機能は既に `SimpleSchema.Type.DateTime` に入っています。\n\n`SimpleSchema` ビヘイビアが要求する関数は、以下の通りです。\n\n```elixir\n@callback schema(opts :: Keyword.t) :: simple_schema\n@callback from_json(schema :: simple_schema, json :: any, opts :: Keyword.t) :: {:ok, any} | {:error, any}\n@callback to_json(schema :: simple_schema, value :: any, opts :: Keyword.t) :: {:ok, any} | {:error, any}\n```\n\n`schema/1` で、そのモジュールが要求するシンプルスキーマを定義します。\n\n`from_json/3` で、`value` を任意の型に変換して返します。\n`value` は `schema/1` で返したシンプルスキーマによる検証が済んでいて、例えば上記の `DateTimeSchema.from_json/3` に渡された `value` は、`{:string, format: :datetime}` で検証されています。\nそのため `value` が文字列であり、`:datetime` のフォーマットであることが保証されています。\n\nただし、`SimpleSchema.from_json/2` に `optimistic: true` が指定されていた場合、検証を行いません。この場合、正しい値を渡す責任はユーザにあります。\n\n`to_json/3` で、変換された値をシンプルスキーマの満たす文字列に変換します。\n`from_json/3` と逆の変換を行います。\nこの関数は `SimpleSchema.to_json/2` の内部で利用される関数なので、不要であれば常に `{:error, \"not implemented\"}` でも構いません。\n\n## `defschema/1`\n\n`defschema/1` は、`defstruct/1` による構造体の定義と、`SimpleSchema` ビヘイビアの実装を行います。\n\n```elixir\ndefmodule Person do\n  import SimpleSchema, only: [defschema: 1]\n\n  defschema [\n    name: :string,\n    age: {:integer, minimum: 0},\n  ]\nend\n```\n\nこのコードは、以下の様に変換されます。\n\n```elixir\ndefmodule Person do\n  @enforce_keys [:name, :age]\n  defstruct [:name, :age]\n\n  @behaviour SimpleSchema\n\n  @impl SimpleSchema\n  def schema(_opts) do\n    %{\n      name: :string,\n      age: {:integer, minimum: 0},\n    }\n  end\n\n  @impl SimpleSchema\n  def from_json(schema, value, _opts) do\n    SimpleSchema.Type.json_to_struct(__MODULE__, schema, value)\n  end\n\n  @impl SimpleSchema\n  def to_json(schema, value, _opts) do\n    SimpleSchema.Type.struct_to_json(__MODULE__, schema, value)\n  end\nend\n```\n\n`schema/1` で、`:name` と `:age` を持つマップとしてシンプルスキーマを定義しています。\n渡された JSON のオブジェクトがこのシンプルスキーマの構造になっているかを検証した後、`Person.from_json/3` を呼び出して、`value` を `Person` 構造体に変換しています。\nJSON のオブジェクトを特定の構造体に変換するためのヘルパーとして `SimpleSchema.Type.json_to_struct/3` があるので、これを使うと簡単に変換できます。","ref":"SimpleSchema.html#module-%E3%83%A1%E3%82%BF%E6%83%85%E5%A0%B1"},{"type":"macro","title":"SimpleSchema.defschema/2","doc":"Generate a struct and implement SimpleSchema behaviour by the specified schema.\n\n```\ndefmodule MySchema do\n  defschema [\n    username: {:string, min_length: 4},\n    email: {:string, default: \"\", optional: true, format: :email},\n  ]\nend\n```\n\nis converted to:\n\n```\ndefmodule MySchema do\n  @enforce_keys [:username]\n  defstruct [:username, email: \"\"]\n\n  @behaviour SimpleSchema\n\n  @simple_schema %{\n    username: {:string, min_length: 4},\n    email: {:string, default: \"\", optional: true, format: :email},\n  }\n\n  @impl SimpleSchema\n  def schema(opts) do\n    {@simple_schema, opts}\n  end\n\n  @impl SimpleSchema\n  def from_json(schema, value, _opts) do\n    SimpleSchema.Type.json_to_struct(__MODULE__, schema, value)\n  end\n\n  @impl SimpleSchema\n  def to_json(schema, value, _opts) do\n    SimpleSchema.Type.struct_to_json(__MODULE__, schema, value)\n  end\nend\n```","ref":"SimpleSchema.html#defschema/2"},{"type":"function","title":"SimpleSchema.from_json/3","doc":"Convert JSON value to a simple schema value.\n\nJSON value is validated before it is converted to a simple schema value.\n\nIf `optimistic: true` is specified in `opts`, JSON value is not validated before it is converted.","ref":"SimpleSchema.html#from_json/3"},{"type":"callback","title":"SimpleSchema.from_json/3","doc":"","ref":"SimpleSchema.html#c:from_json/3"},{"type":"function","title":"SimpleSchema.from_json!/3","doc":"","ref":"SimpleSchema.html#from_json!/3"},{"type":"callback","title":"SimpleSchema.schema/1","doc":"","ref":"SimpleSchema.html#c:schema/1"},{"type":"function","title":"SimpleSchema.to_json/3","doc":"Convert a simple schema value to JSON value.\n\nIf `optimistic: true` is specified in `opts`, JSON value is not validated after it is converted.\nOtherwise, JSON value is validated after it is converted.","ref":"SimpleSchema.html#to_json/3"},{"type":"callback","title":"SimpleSchema.to_json/3","doc":"","ref":"SimpleSchema.html#c:to_json/3"},{"type":"function","title":"SimpleSchema.to_json!/3","doc":"","ref":"SimpleSchema.html#to_json!/3"},{"type":"type","title":"SimpleSchema.simple_schema/0","doc":"","ref":"SimpleSchema.html#t:simple_schema/0"},{"type":"module","title":"SimpleSchema.Schema","doc":"A module to convert a simple schema to JSON Schema\n\nBasic:\n\n```\niex> schema = %{name: :string,\n...>            value: {:integer, optional: true},\n...>            array: [:string],\n...>            map: {%{x: :integer, y: :integer}, optional: true},\n...>            param: {:any, optional: true}}\niex> SimpleSchema.Schema.to_json_schema(schema)\n%{\n  \"type\" => \"object\",\n  \"required\" => [\"array\", \"name\"],\n  \"additionalProperties\" => false,\n  \"properties\" => %{\n    \"name\" => %{\"type\" => \"string\"},\n    \"value\" => %{\"type\" => \"integer\"},\n    \"array\" => %{\n      \"type\" => \"array\",\n      \"items\" => %{\"type\" => \"string\"},\n    },\n    \"map\" => %{\n      \"type\" => \"object\",\n      \"required\" => [\"x\", \"y\"],\n      \"additionalProperties\" => false,\n      \"properties\" => %{\n        \"x\" => %{\"type\" => \"integer\"},\n        \"y\" => %{\"type\" => \"integer\"},\n      },\n    },\n    \"param\" => %{\n      \"type\" => [\"array\", \"boolean\", \"integer\", \"null\", \"number\", \"object\", \"string\"],\n    },\n  },\n}\n```\n\nWith restrictions:\n\n```\niex> schema = %{name: {:string, min_length: 8},\n...>            value: {:integer, optional: true, nullable: true, maximum: 10},\n...>            array: {[{:string, enum: [\"aaa\", \"bbb\"]}], min_items: 1}}\niex> SimpleSchema.Schema.to_json_schema(schema)\n%{\n  \"type\" => \"object\",\n  \"required\" => [\"array\", \"name\"],\n  \"additionalProperties\" => false,\n  \"properties\" => %{\n    \"name\" => %{\n      \"type\" => \"string\",\n      \"minLength\" => 8,\n    },\n    \"value\" => %{\n      \"type\" => [\"integer\", \"null\"],\n      \"maximum\" => 10,\n    },\n    \"array\" => %{\n      \"type\" => \"array\",\n      \"minItems\" => 1,\n      \"items\" => %{\n        \"type\" => \"string\",\n        \"enum\" => [\n          \"aaa\",\n          \"bbb\",\n        ],\n      }\n    },\n  },\n}\n```","ref":"SimpleSchema.Schema.html"},{"type":"function","title":"SimpleSchema.Schema.from_json/2","doc":"Convert validated JSON to a simple schema value.\n\nIf validation is passed, The JSON key should be all known atom except `:any` type.\nSo the key can be converted by `String.to_existing_atom/1`.\n\n    iex> schema = %{foo: %{bar: :integer}}\n    iex> SimpleSchema.Schema.from_json(schema, %{\"foo\" => %{\"bar\" => 10}})\n    {:ok, %{foo: %{bar: 10}}}\n\nHowever, `:any` can contains an arbitrary key, so do not convert a value of `:any`.\n\n    iex> schema = %{foo: :any}\n    iex> SimpleSchema.Schema.from_json(schema, %{\"foo\" => %{\"bar\" => 10}})\n    {:ok, %{foo: %{\"bar\" => 10}}}","ref":"SimpleSchema.Schema.html#from_json/2"},{"type":"function","title":"SimpleSchema.Schema.simple_schema_implemented?/1","doc":"","ref":"SimpleSchema.Schema.html#simple_schema_implemented?/1"},{"type":"function","title":"SimpleSchema.Schema.to_json/2","doc":"Convert a simple schema value to JSON value.","ref":"SimpleSchema.Schema.html#to_json/2"},{"type":"function","title":"SimpleSchema.Schema.to_json_schema/2","doc":"","ref":"SimpleSchema.Schema.html#to_json_schema/2"},{"type":"type","title":"SimpleSchema.Schema.any_type/0","doc":"","ref":"SimpleSchema.Schema.html#t:any_type/0"},{"type":"type","title":"SimpleSchema.Schema.array_type/0","doc":"","ref":"SimpleSchema.Schema.html#t:array_type/0"},{"type":"type","title":"SimpleSchema.Schema.boolean_type/0","doc":"","ref":"SimpleSchema.Schema.html#t:boolean_type/0"},{"type":"type","title":"SimpleSchema.Schema.integer_type/0","doc":"","ref":"SimpleSchema.Schema.html#t:integer_type/0"},{"type":"type","title":"SimpleSchema.Schema.map_type/0","doc":"","ref":"SimpleSchema.Schema.html#t:map_type/0"},{"type":"type","title":"SimpleSchema.Schema.module_type/0","doc":"","ref":"SimpleSchema.Schema.html#t:module_type/0"},{"type":"type","title":"SimpleSchema.Schema.null_type/0","doc":"","ref":"SimpleSchema.Schema.html#t:null_type/0"},{"type":"type","title":"SimpleSchema.Schema.number_type/0","doc":"","ref":"SimpleSchema.Schema.html#t:number_type/0"},{"type":"type","title":"SimpleSchema.Schema.opts/0","doc":"","ref":"SimpleSchema.Schema.html#t:opts/0"},{"type":"type","title":"SimpleSchema.Schema.simple_schema/0","doc":"","ref":"SimpleSchema.Schema.html#t:simple_schema/0"},{"type":"type","title":"SimpleSchema.Schema.string_type/0","doc":"","ref":"SimpleSchema.Schema.html#t:string_type/0"},{"type":"module","title":"SimpleSchema.Type","doc":"","ref":"SimpleSchema.Type.html"},{"type":"function","title":"SimpleSchema.Type.json_to_struct/3","doc":"","ref":"SimpleSchema.Type.html#json_to_struct/3"},{"type":"function","title":"SimpleSchema.Type.struct_to_json/3","doc":"","ref":"SimpleSchema.Type.html#struct_to_json/3"},{"type":"module","title":"SimpleSchema.Type.DateTime","doc":"","ref":"SimpleSchema.Type.DateTime.html"},{"type":"module","title":"SimpleSchema.Type.Unique","doc":"","ref":"SimpleSchema.Type.Unique.html"},{"type":"behaviour","title":"SimpleSchema.Validator","doc":"","ref":"SimpleSchema.Validator.html"},{"type":"function","title":"SimpleSchema.Validator.validate/5","doc":"","ref":"SimpleSchema.Validator.html#validate/5"},{"type":"callback","title":"SimpleSchema.Validator.validate/5","doc":"","ref":"SimpleSchema.Validator.html#c:validate/5"},{"type":"module","title":"SimpleSchema.Validator.ExJsonSchema","doc":"","ref":"SimpleSchema.Validator.ExJsonSchema.html"}],"content_type":"text/markdown"}